@@ -5,9 +5,9 @@
 ###
 ### Snapshot map (spplot) of an sts-object or matrix of counts
 ###
-### Copyright (C) 2013-2014 Sebastian Meyer
-### $Revision: 1051 $
-### $Date: 2014-10-06 22:40:52 +0200 (Mon, 06 Oct 2014) $
+### Copyright (C) 2013-2014,2016 Sebastian Meyer
+### $Revision: 1642 $
+### $Date: 2016-03-16 13:37:42 +0100 (Mit, 16. Mär 2016) $
 ################################################################################
 
 ## x: "sts" or (simulated) matrix of counts
@@ -26,11 +26,18 @@ stsplot_space <- function (x, tps = NULL, map = x@map,
                            main = NULL, labels = FALSE,
                            at = 10, col.regions = NULL,
                            colorkey = list(space="bottom", labels=list(at=at)),
-                           total.args = NULL, sp.layout = NULL, ...)
+                           total.args = NULL, 
+                           gpar.missing = list(col="darkgrey", lty=2, lwd=2),
+                           sp.layout = NULL,
+                           xlim = bbox(map)[1, ], ylim = bbox(map)[2, ], ...)
 {
     counts <- if (inherits(x, "sts")) observed(x) else x
-    map <- map[colnames(x),] # assure same order
-
+    if (length(map) == 0L) stop("no map")
+    if (is.null(colnames(counts)))
+        stop("need 'colnames(x)' (to be matched against 'row.names(map)')")
+    if (!all(colnames(counts) %in% row.names(map)))
+        stop("incomplete 'map'; ensure that 'all(colnames(x) %in% row.names(map))'")
+    
     ## compute data to plot
     ncases <- getCumCounts(counts, tps)
     total <- sum(ncases)
@@ -42,13 +49,10 @@ stsplot_space <- function (x, tps = NULL, map = x@map,
     }
 
     ## add ncases to map@data
-    if (inherits(map, "SpatialPolygonsDataFrame")) {
-        map$ncases <- ncases
-    } else {
-        map <- SpatialPolygonsDataFrame(map, as.data.frame(ncases),
-                                        match.ID=FALSE)
-    }
-
+    map <- as(map, "SpatialPolygonsDataFrame")
+    map$ncases <- NA_real_
+    map$ncases[match(colnames(counts),row.names(map))] <- ncases
+    
     ## default main title
     if (is.null(main) && inherits(x, "sts"))
         main <- stsTimeRange2text(x, tps)
@@ -68,6 +72,11 @@ stsplot_space <- function (x, tps = NULL, map = x@map,
         colorkey <- modifyList(eval(formals()$colorkey), colorkey)
 
     ## automatic additions to sp.layout (region labels and total)
+    if (is.list(gpar.missing) && any(is.na(map$ncases))) {
+        layout.missing <- c(list("sp.polygons", obj=map[is.na(map$ncases),]),
+                            gpar.missing)
+        sp.layout <- c(sp.layout, list(layout.missing))
+    }
     if (!is.null(layout.labels <- layout.labels(map, labels))) {
         sp.layout <- c(sp.layout, list(layout.labels))
     }
@@ -85,9 +94,9 @@ stsplot_space <- function (x, tps = NULL, map = x@map,
     }
 
     ## generate the spplot()
-    args <- list(quote(map), "ncases", main=main,
+    args <- list(quote(map[!is.na(map$ncases),]), "ncases", main=main,
                  col.regions=col.regions, at=at, colorkey=colorkey,
-                 sp.layout=sp.layout, quote(...))
+                 sp.layout=sp.layout, xlim=xlim, ylim=ylim, quote(...))
     do.call("spplot", args)
 }
 
@@ -135,12 +144,13 @@ getCountIntervals <- function (nInt, data, trafo=scales::sqrt_trans(), ...) {
     at
 }
 
-stsTime2text <- function (stsObj, tps=TRUE)
-    paste(year(stsObj)[tps], epochInYear(stsObj)[tps], sep="/")
+stsTime2text <- function (stsObj, tps=TRUE, fmt="%i/%i") {
+    sprintf(fmt, year(stsObj)[tps], epochInYear(stsObj)[tps])
+}
 
-stsTimeRange2text <- function (stsObj, tps=NULL)
+stsTimeRange2text <- function (stsObj, tps=NULL, fmt="%i/%i", sep=" - ")
 {
     tpsRange <- if (is.null(tps)) c(1, nrow(stsObj)) else range(tps)
-    tpsRangeYW <- stsTime2text(stsObj, tpsRange)
-    paste(unique(tpsRangeYW), collapse=" - ")
+    tpsRangeYW <- stsTime2text(stsObj, tps=tpsRange, fmt=fmt)
+    paste0(unique(tpsRangeYW), collapse=sep)
 }
