@@ -5,50 +5,57 @@
 ###
 ### Time series plot for sts-objects
 ###
-### Copyright (C) 2007-2013 Michael Hoehle
-### $Revision: 1003 $
-### $Date: 2014-09-07 11:28:42 +0200 (Sun, 07 Sep 2014) $
+### Copyright (C) 2007-2014 Michael Hoehle, 2013-2016 Sebastian Meyer
+### $Revision: 1676 $
+### $Date: 2016-04-01 11:42:40 +0200 (Fre, 01. Apr 2016) $
 ################################################################################
 
 
 ######################################################################
-# stsplot_time sets the scene and calls stsplot_time1 for each unit
+# stsplot_time() sets the scene and calls either stsplot_time_as1()
+# or stsplot_time1() for each unit
 ######################################################################
 
-stsplot_time <- function(x, units,
-                         method=x@control$name, disease=x@control$data,
-                         as.one=FALSE, same.scale=TRUE, par.list=list(), ...)
+stsplot_time <- function(x, units = NULL,
+                         as.one = FALSE, same.scale = TRUE,
+                         par.list = list(), ...)
 {
-  #Extract
   observed <- x@observed
-  population <- x@populationFrac
-  binaryTS <- x@multinomialTS
-  nUnits <- ncol(observed)
+  if (is.null(units)) # plot all units
+    units <- seq_len(ncol(observed))
+  nUnits <- length(units)
 
-  #multivariate time series
-  if(nUnits > 1){
-    if (missing(units))
-      units <- seq_len(nUnits)
-    if(as.one) { # all areas in one plot
-      stop("this type of plot is currently not implemented")
-    } else {
-      par.list <- modifyList(
+  #graphical parameters
+  if (is.list(par.list)) {
+    if (nUnits > 1 && !as.one) {
+      par.list <- modifyList( #default: reduced margins and mfrow panels
         list(mar = c(5,4,1,1), mfrow = magic.dim(nUnits)),
         par.list)
-
-      #set window size
+    } else {
+      par.list$mfrow <- NULL #no mf formatting..
+    }
+    if (length(par.list) > 0) {
       oldpar <- par(par.list)
-
-      #All plots on same scale? If yes, then check if a scale
-      #is already specified using the ylim argument
+      on.exit(par(oldpar))
+    }
+  }
+  
+  if (nUnits == 1L) { # a single time-series plot
+    stsplot_time1(x = x, k = units, ...)
+  } else { # multiple time series
+    if (as.one) { # all time series in one plot
+      stsplot_time_as1(x, units = units, ...)
+    } else { # each time series in a separate plot
       args <- list(...)
-      if(same.scale) {
-        if (is.null(args$ylim)) {
-            max <- if (binaryTS) {
-                max(ifelse(population == 0, 0,
-                           pmax(observed,x@upperbound,na.rm=TRUE)/population))
-            } else max(observed,x@upperbound,na.rm=TRUE)
-            args$ylim <- c(-1/20*max, max)
+      if(same.scale) { # compute suitable ylim if not specified
+        if (is.null(args[["ylim"]])) {
+            ymax <- if (x@multinomialTS) {
+                max(0, pmax(observed,x@upperbound,na.rm=TRUE)/x@populationFrac,
+                    na.rm=TRUE)
+            } else {
+                max(observed,x@upperbound,na.rm=TRUE)
+            }
+            args$ylim <- c(-1/20*ymax, ymax)
         }
       } else {
         args$ylim <- NULL
@@ -59,23 +66,57 @@ stsplot_time <- function(x, units,
         argsK <- modifyList(args, list(x=x, k=k, main="", legend.opts=NULL),
                             keep.null = TRUE)
         do.call("stsplot_time1",args=argsK)
-        title(main=colnames(observed)[k], line=-1)
+        title(main=if (is.character(k)) k else colnames(observed)[k], line=-1)
       }
-      
-      #reset graphical params
-      par(oldpar)
     }
-  } else {  #univariate time series
-    par.list$mfrow <- NULL #no mf formatting..
-    oldpar <- par(par.list)
-    stsplot_time1(x=x, ...)
-    par(oldpar)
   }
+  
   invisible()
 }
 
 
-### work-horse which produces the time series plot with formatted x-axis
+## a simple matplot of observed counts from all/selected units, with a legend
+
+stsplot_time_as1 <- function (x, units = NULL,
+    type = "l", lty = 1:5, lwd = 1, col = 1:6,
+    epochsAsDate = x@epochAsDate, xaxis.tickFreq = list("%Q"=atChange),
+    xaxis.labelFreq = xaxis.tickFreq, xaxis.labelFormat = "%G\n\n%OQ",
+    xlab = "time", ylab = "No. infected", legend.opts = list(), ...)
+{
+    observed <- x@observed
+    if (x@multinomialTS) {
+        observed <- ifelse(x@populationFrac != 0, observed/x@populationFrac, 0)
+    }
+    if (!is.null(units))
+        observed <- observed[, units, drop = FALSE]
+
+    ## basic plot
+    opar <- par(bty = "n", xaxt = "n")  # a formatted time axis is added below
+    matplot(observed, type = type, lty = lty, lwd = lwd, col = col,
+            xlab = xlab, ylab = ylab, ...)
+    par(opar)
+
+    ## add time axis
+    xaxis.line <- !epochsAsDate || grepl("\n", xaxis.labelFormat)
+    addFormattedXAxis(x = x, epochsAsDate = epochsAsDate,
+        xaxis.tickFreq = xaxis.tickFreq, xaxis.labelFreq = xaxis.labelFreq,
+        xaxis.labelFormat = xaxis.labelFormat)  # line = 1
+
+    ## add legend
+    if (is.list(legend.opts)) {
+        legend.opts <- modifyList(
+            list(x = "top", legend = colnames(observed), lty = lty, lwd = lwd,
+                 col = col, ncol = magic.dim(ncol(observed))[2L],
+                 bty = "n"),
+            legend.opts)
+        do.call("legend", legend.opts)
+    }
+    
+    invisible()
+}
+
+
+### work-horse which produces a single time series plot with formatted x-axis
 
 stsplot_time1 <- function(
     x, k=1, ylim=NULL, axes=TRUE, xaxis.tickFreq=list("%Q"=atChange),
@@ -84,20 +125,16 @@ stsplot_time1 <- function(
     type="s", lty=c(1,1,2), col=c(NA,1,4), lwd=c(1,1,1),
     outbreak.symbol=list(pch=3, col=3, cex=1, lwd=1),
     alarm.symbol=list(pch=24, col=2, cex=1, lwd=1),
-    legend.opts=list(x="top", legend=NULL, lty=NULL, pch=NULL, col=NULL),
+    legend.opts=list(),
     dx.upperbound=0L, hookFunc=function(){}, .hookFuncInheritance=function() {}, ...)
 {
-
+  stopifnot(length(k) == 1, is.character(k) || k != 0)
+  
   #Extract slots -- depending on the algorithms: x@control$range
   observed   <- x@observed[,k]
   state      <- x@state[,k]
   alarm      <- x@alarm[,k]
   upperbound <- x@upperbound[,k]
-  hasAlarm   <- all(!is.na(alarm))
-  startyear <-  x@start[1]
-  firstweek <-  x@start[2]
-  method <-     x@control$name
-  disease <-    x@control$data
   population <- x@populationFrac[,k]
   binaryTS <- x@multinomialTS
 
@@ -110,14 +147,13 @@ stsplot_time1 <- function(
     if (ylab == "No. infected") { ylab <- "Proportion infected" }
   }
   
-   ##### Handle the NULL arguments ######################################
-  if (is.null(main)) {
-    #If no surveillance algorithm has been run
-    if (length(x@control) != 0) {
-#      main = paste("Surveillance using ", as.character(method),sep="")
-        action = switch(class(x), "sts"="surveillance","stsNC"="nowcasting","stsBP"="backprojection")
-        main = paste(action," using ", as.character(method),sep="") 
-    }
+  ##### Handle the NULL arguments ######################################
+  if (is.null(main) && length(x@control) > 0) {
+    #a surveillance algorithm has been run
+    action <- switch(class(x), "sts" = "surveillance",
+                     "stsNC" = "nowcasting","stsBP" = "backprojection")
+    method <- x@control$name
+    main <- paste0(action, " using ", method)
   }
 
   # control where the highest value is
@@ -151,13 +187,13 @@ stsplot_time1 <- function(
     lines(x=xstuff,y=ystuff,type=type,lty=lty[-c(1:2)],col=col[-c(1:2)],lwd=lwd[-c(1:2)],...)
   }
   
-  #Draw outbreak symbols
+  #Draw alarm symbols
   alarmIdx <- which(!is.na(alarm) & (alarm == 1))
   if (length(alarmIdx)>0) {
     matpoints( alarmIdx, rep(-1/40*ylim[2],length(alarmIdx)), pch=alarm.symbol$pch, col=alarm.symbol$col, cex= alarm.symbol$cex, lwd=alarm.symbol$lwd)
   }
   
-  #Draw alarm symbols
+  #Draw outbreak symbols
   stateIdx <- which(state == 1)
   if (length(stateIdx)>0) {
     matpoints( stateIdx, rep(-1/20*ylim[2],length(stateIdx)), pch=outbreak.symbol$pch, col=outbreak.symbol$col,cex = outbreak.symbol$cex,lwd=outbreak.symbol$lwd)
@@ -165,22 +201,28 @@ stsplot_time1 <- function(
 
   #Label x-axis 
   if(xaxis.dates & axes) {
-    addFormattedXAxis(x, epochsAsDate, observed, firstweek,xaxis.labelFormat,xaxis.tickFreq,xaxis.labelFreq,...)
+    addFormattedXAxis(x = x, epochsAsDate = epochsAsDate, xaxis.tickFreq = xaxis.tickFreq,
+                      xaxis.labelFreq = xaxis.labelFreq, xaxis.labelFormat = xaxis.labelFormat,
+                      ...)
   }
   #Label y-axis
   if (axes) {
     axis( side=2 ,...)#cex=cex, cex.axis=cex.axis)
   }
 
-  if(is.list(legend.opts)) {
-    #Fill empty (mandatory) slots in legend.opts list
-    if (is.null(legend.opts$x)) legend.opts$x <- "topleft"
-    if (is.null(legend.opts$lty)) legend.opts$lty <- c(lty[1],lty[3],NA,NA)
-    if (is.null(legend.opts$col)) legend.opts$col <- c(col[2],col[3],outbreak.symbol$col,alarm.symbol$col)
-    if (is.null(legend.opts$pch)) legend.opts$pch <- c(NA,NA,outbreak.symbol$pch,alarm.symbol$pch)
-    if (is.null(legend.opts$legend)) {
-      legend.opts$legend <- c("Infected", "Threshold","Outbreak","Alarm" )
-    }
+  doLegend <- if (missing(legend.opts)) {
+      length(stateIdx) + length(alarmIdx) > 0 || any(upperbound > 0, na.rm = TRUE)
+  } else {
+      is.list(legend.opts)
+  }
+  if(doLegend) {
+      legend.opts <- modifyList(
+          list(x = "top",
+               lty = c(lty[1],lty[3],NA,NA),
+               col = c(col[2],col[3],outbreak.symbol$col,alarm.symbol$col),
+               pch = c(NA,NA,outbreak.symbol$pch,alarm.symbol$pch),
+               legend = c("Infected", "Threshold", "Outbreak", "Alarm")),
+          legend.opts)
     #Make the legend
     do.call("legend",legend.opts)
   }
@@ -218,20 +260,15 @@ stsplot_alarm <- function(
   state      <- x@state[,k]
   alarm      <- x@alarm[,k]
   upperbound <- x@upperbound[,k]
-  hasAlarm   <- all(!is.na(alarm))
-  startyear <-  x@start[1]
-  firstweek <-  x@start[2]
-  method <-     x@control$name
-  disease <-    x@control$data
   ylim <- c(0.5, ncol(x))
   
   ##### Handle the NULL arguments ######################################
-  if (is.null(main)) {
-    #If no surveillance algorithm has been run
-    if (length(x@control) != 0) {
-     # main = paste("Analysis of ", as.character(disease), " using ",
-      main = paste("Surveillance using ", as.character(method),sep="") 
-    }
+  if (is.null(main) && length(x@control) > 0) {
+    #a surveillance algorithm has been run
+    action <- switch(class(x), "sts" = "surveillance",
+                     "stsNC" = "nowcasting","stsBP" = "backprojection")
+    method <- x@control$name
+    main <- paste0(action, " using ", method)
   }
  
   #Control what axis style is used
@@ -263,7 +300,9 @@ stsplot_alarm <- function(
 
   #Label of x-axis 
   if(xaxis.dates){
-    addFormattedXAxis(x, epochsAsDate, observed, firstweek, xaxis.labelFormat,xaxis.tickFreq,xaxis.labelFreq,...)
+    addFormattedXAxis(x = x, epochsAsDate = epochsAsDate, xaxis.tickFreq = xaxis.tickFreq,
+                      xaxis.labelFreq = xaxis.labelFreq, xaxis.labelFormat = xaxis.labelFormat,
+                      ...)
   }
   axis( side=2, at=1:ncol(x),cex.axis=cex.yaxis, labels=colnames(x),las=2)
 
@@ -304,18 +343,23 @@ at2ndChange <- function(x,xm1) {
 }
 
 #Helper function to format the x-axis of the time series
-addFormattedXAxis <- function(x, epochsAsDate, observed, firstweek, xaxis.labelFormat, xaxis.tickFreq,xaxis.labelFreq,...) {
+addFormattedXAxis <- function(x, epochsAsDate = FALSE,
+                              xaxis.tickFreq = list("%Q"=atChange),
+                              xaxis.labelFreq = xaxis.tickFreq, xaxis.labelFormat = "%G\n\n%OQ",
+                              ...) {
   
   #Old style if there are no Date objects
   if (!epochsAsDate) {
     #Declare commonly used variables.
+    nTime <- nrow(x)
     startyear <-  x@start[1]
+    firstweek <-  x@start[2]
     
     if (x@freq ==52) {  #Weekly epochs are the most supported 
       # At which indices to put the "at" tick label. This will
       # be exactly those week numbers where the new quarter begins: 1, 14, 27 and 40 + i*52.
       # Note that week number and index is not the same due to the "firstweek" argument
-      weeks <- 1:length(observed) + (firstweek-1)
+      weeks <- seq_len(nTime) + (firstweek-1)
       noYears <- ceiling(max(weeks)/52)
       quarterStarts <- rep( (0:(noYears))*52, each=4) + rep( c(1,14,27,40), noYears+1)
       weeks <- subset(weeks, !is.na(match(weeks,quarterStarts)))
@@ -330,15 +374,15 @@ addFormattedXAxis <- function(x, epochsAsDate, observed, firstweek, xaxis.labelF
       #Computed axis labels -- add quarters (this is the old style)
       labels.week <- paste(year,"\n\n",quarter,sep="")
       #Make the line. Use lwd.ticks to get full line but no marks.
-      axis( side=1,labels=FALSE,at=c(1,length(observed)),lwd.ticks=0,line=1,...) 
+      axis( side=1,labels=FALSE,at=c(1,nTime),lwd.ticks=0,line=1,...) 
       axis( at=weekIdx[which(quarter != "I")] , labels=labels.week[which(quarter != "I")] , side=1, line = 1 ,...)       
       #Bigger tick marks at the first quarter (i.e. change of the year)
       at <- weekIdx[which(quarter == "I")]
-      axis( at=at, labels=rep(NA,length(at)), side=1, line = 1 ,tcl=2*par()$tcl)
+      axis( at=at, labels=rep(NA,length(at)), side=1, line = 1 ,tcl=2*par("tcl"))
       
     } else { ##other frequency (not really supported)
       #A label at each unit
-      myat.unit <- seq(firstweek,length.out=length(observed) )
+      myat.unit <- seq(firstweek,length.out=nTime)
       
       # get the right year order
       month <- (myat.unit-1) %% x@freq + 1
@@ -347,30 +391,30 @@ addFormattedXAxis <- function(x, epochsAsDate, observed, firstweek, xaxis.labelF
       mylabels.unit <- paste(year,"\n\n", (myat.unit-1) %% x@freq + 1,sep="")
       
       #Add axis
-      axis( at=(1:length(observed)), labels=NA, side=1, line = 1, ...) 
-      axis( at=(1:length(observed))[month==1], labels=mylabels.unit[month==1] , side=1, line = 1 ,...)
+      axis( at=seq_len(nTime), labels=NA, side=1, line = 1, ...) 
+      axis( at=seq_len(nTime)[month==1], labels=mylabels.unit[month==1] , side=1, line = 1 ,...)
       #Bigger tick marks at the first unit
-      at <- (1:length(observed))[(myat.unit - 1) %% x@freq == 0]
-      axis( at=at, labels=rep(NA,length(at)), side=1, line = 1 ,tcl=2*par()$tcl)
+      at <- seq_len(nTime)[(myat.unit - 1) %% x@freq == 0]
+      axis( at=at, labels=rep(NA,length(at)), side=1, line = 1 ,tcl=2*par("tcl"))
     } 
   } else {   
     ################################################################
     #epochAsDate -- experimental functionality to handle ISO 8601
     ################################################################
     
-    dates <- epoch(x)
+    dates <- epoch(x, as.Date = TRUE)
     #make one which has one extra element at beginning with same spacing
     datesOneBefore <- c(dates[1]-(dates[2]-dates[1]),dates)
     
     #Make the line. Use lwd.ticks to get full line but no marks.
-    axis( side=1,labels=FALSE,at=c(1,length(observed)),lwd.ticks=0,...) 
+    axis( side=1,labels=FALSE,at=c(1,length(dates)),lwd.ticks=0,...) 
     
     ###Make the ticks (depending on the selected level).###
-    tcl <- par()$tcl
+    tcl <- par("tcl")
     tickFactors <- surveillance.options("stsTickFactors")
     
     #Loop over all pairs in the xaxis.tickFreq list
-     for (i in seq(xaxis.tickFreq)) {
+     for (i in seq_along(xaxis.tickFreq)) {
       format <- names(xaxis.tickFreq)[i]
       xm1x <- as.numeric(formatDate(datesOneBefore,format))                         
       idx <- xaxis.tickFreq[[i]](x=xm1x[-1],xm1=xm1x[1])
@@ -378,7 +422,7 @@ addFormattedXAxis <- function(x, epochsAsDate, observed, firstweek, xaxis.labelF
       #Find tick size by table lookup
       tclFactor <- tickFactors[pmatch(format, names(tickFactors))]
       if (is.na(tclFactor)) { 
-        warning("No tcl factor found for",format,". Setting it at 1") 
+        warning("no \"tcl\" factor found for \"", format ,"\" -> setting it to 1")
         tclFactor <- 1
       }
       axis(1,at=idx, labels=NA,tcl=tclFactor*tcl,...)
@@ -387,7 +431,7 @@ addFormattedXAxis <- function(x, epochsAsDate, observed, firstweek, xaxis.labelF
     ###Make the labels (depending on the selected level)###
     if (!is.null(xaxis.labelFormat)) {
       labelIdx <- NULL
-      for (i in seq(xaxis.labelFreq)) {
+      for (i in seq_along(xaxis.labelFreq)) {
         format <- names(xaxis.labelFreq)[i]
         xm1x <- as.numeric(formatDate(datesOneBefore,format))                         
         labelIdx <- c(labelIdx,xaxis.labelFreq[[i]](x=xm1x[-1],xm1=xm1x[1]))
@@ -396,7 +440,7 @@ addFormattedXAxis <- function(x, epochsAsDate, observed, firstweek, xaxis.labelF
       #Format labels (if any) for the requested subset
       if (length(labelIdx)>0) {
           labels <- rep(NA,nrow(x))
-          labels[labelIdx] <- formatDate(epoch(x)[labelIdx],xaxis.labelFormat)
+          labels[labelIdx] <- formatDate(dates[labelIdx],xaxis.labelFormat)
           axis(1,at=1:nrow(x), labels=labels,tick=FALSE,...)
       }
     }
