@@ -1,7 +1,7 @@
 ######################################################################
 #  Demo of the code used in the book chapter
-#  Hoehle, M. and A. Mazick, A. (2009) Aberration detection in R
-#  illustrated by Danish mortality monitoring,  Book chapter to appear in
+#  Hoehle, M. and A. Mazick, A. (2010) Aberration detection in R
+#  illustrated by Danish mortality monitoring,  Book chapter in
 #  T. Kass-Hout and X. Zhang (Eds.) Biosurveillance: A Health Protection
 #  Priority, CRC Press.
 #
@@ -20,13 +20,14 @@ library("surveillance")
 #Load Danish mortality data (see book chapter for CSV reading")
 data("momo")
 
-#Create a plot of the data as in Figure. 1 of the book chapter
-plot(momo[year(momo)>=2000,],ylab="No. of deaths",par.list=list(mar=c(4,2.2,2,1),cex.axis=1.5), type=observed ~ time | unit, col=c(gray(0.3),NA,NA),xaxis.units=FALSE,xlab="time (weeks)")
+#Create a plot of the data as in Figure. 1 of the book chapter.
+#Note: The year is determined by the ISO week, not the date
+plot(momo[year(momo)>=2000,],ylab="No. of deaths",par.list=list(mar=c(4,2.2,2,1),cex.axis=1.5), type=observed ~ time | unit, col=c(gray(0.3),NA,NA),xaxis.tickFreq=list("%G"=atChange),xaxis.labelFormat="%G",xlab="time (weeks)")
 
 
 par(mfrow=c(1,2),mar=c(4,4,2,1))
-plot(momo,ylab="No. of deaths",xlab="time (weeks)",legend.opts=NULL, type=observed ~ time,col=c(gray(0.3),NA,NA))
-plot(momo[,"[0,1)"],xlab="time (weeks)",ylab="No. of deaths",legend.opts=NULL,col=c(gray(0.3),NA,NA))
+plot(momo,ylab="No. of deaths",xlab="time (weeks)",legend.opts=NULL, type=observed ~ time,col=c(gray(0.3),NA,NA),xaxis.tickFreq=list("%G"=atChange,"%m"=atChange),xaxis.labelFreq=list("%G"=atChange),xaxis.labelFormat="%G")
+plot(momo[,"[0,1)"],xlab="time (weeks)",ylab="No. of deaths",legend.opts=NULL,col=c(gray(0.3),NA,NA),xaxis.tickFreq=list("%G"=atChange,"%m"=atChange),xaxis.labelFreq=list("%G"=atChange),xaxis.labelFormat="%G")
 par(mfrow=c(1,1))
 
 #Monitoring starts in week 40, 2007
@@ -81,14 +82,14 @@ s.far.all <- farrington(momo, control=list(range=phase2,alpha=0.01,b=5,w=4))
 
 
 par(mar=c(4,4,1,1))
-plot(s.far.all,type = alarm ~ time,xlab="time (weeks)",main="",alarm.symbol=list(pch=24,col=1, cex=1),lvl=rep(0,nrow(s.far.all)))
+plot(s.far.all,type = alarm ~ time,xlab="time (weeks)",main="",alarm.symbol=list(pch=24,col=1, cex=1.5),lvl=rep(1,nrow(s.far.all)))
 
 #######################################################################
 #Negative binomial GLM modelling using the population size as covariate
 #######################################################################
 phase1 <- which(year(momo) == 2002 & epochInYear(momo) == 40):(phase2[1]-1)
 momo.df <- as.data.frame(momo)
-m <- glm.nb( `observed.[75,85)` ~ 1 + epoch + sin(2*pi*epochInPeriod) + cos(2*pi*epochInPeriod) + `population.[75,85)`, data=momo.df[phase1,])
+m <- MASS::glm.nb( `observed.[75,85)` ~ 1 + epoch + sin(2*pi*epochInPeriod) + cos(2*pi*epochInPeriod) + `population.[75,85)`, data=momo.df[phase1,])
 mu0 <- predict(m, newdata=momo.df[phase2,],type="response")
 
 
@@ -128,7 +129,7 @@ nSims <- 10 #1000
 simone.TAleq65 <- function(sts, g) {
   observed(sts)[phase2,] <- rnbinom(length(mu0), mu=mu0, size=m$theta)
   one <- glrnb(sts, control=modifyList(control(s.nb), list(c.ARL=g)))
-  return(any(alarms(one)))
+  return(any(alarms(one) > 0))
 }
 
 #Determine run-length using 1000 Monte Carlo samples
@@ -159,7 +160,7 @@ axis(2,at=prob,las=1,cex.axis=0.7)
 legend(x="topright",c("Monte Carlo","Markov chain"), lty=1:2,col=1)
 
 
-m.01 <- glm.nb( `observed.[0,1)` ~ 1 + epoch + `population.[0,1)`+ sin(2*pi*epochInPeriod) + cos(2*pi*epochInPeriod), data=momo.df[phase1,])
+m.01 <- MASS::glm.nb( `observed.[0,1)` ~ 1 + epoch + `population.[0,1)`+ sin(2*pi*epochInPeriod) + cos(2*pi*epochInPeriod), data=momo.df[phase1,])
 mu0 <- predict(m.01, newdata=momo.df[phase2,],type="response")
 
 #Correct for past outbreaks
@@ -198,7 +199,3 @@ lines(1:(nrow(s.far)+1)-0.5, c(mu0.far,tail(mu0.far,n=1)),lwd=3,col=1,lty=1,type
 lines(1:(nrow(s.far)+1)-0.5, c(mu1,tail(mu1,n=1)),col=1,lty=3,lwd=3,type="s")
 legend(x="topright",c(expression(mu[0,t]),expression(mu[1,t]),"NNBA"),col=c(1,1,1),lty=c(1,3,1),horiz=TRUE,bg="white",lwd=c(3,3,1))
 
-
-## demo(biosurvbook)
-
-
